<?php

/**
 * @file
 * Administrative interface for the Mail Editor module.
 */

/**
 * Form builder for the template list.
 *
 * @return array
 */
function mail_edit_list_form() {
  $form['mail_templates'] = mail_edit_list();
  return $form;
}

/**
 * Page callback: List mail templates.
 *
 * @return array
 */
function mail_edit_list() {
  $language_list = mail_edit_language_list();
  $languages_selected = !empty($_SESSION['mail_edit']['languages_selected']) ? $_SESSION['mail_edit']['languages_selected'] : $language_list;
  $module = isset($_SESSION['mail_edit']['module']) ? $_SESSION['mail_edit']['module'] : '-enabled';

  $form['filter'] = mail_edit_list_filter($languages_selected, $module, $language_list);
  $form['filtered_form'] = mail_edit_list_filtered_form($languages_selected, $module, count($language_list) > 1);
  drupal_add_css(drupal_get_path('module', 'mail_edit') . '/mail_edit.admin.css');
  $form['#submit'][] = 'mail_edit_list_submit';
  return $form;
}

/**
 * Filter the mail edit table.
 *
 * @param array $languages_selected
 *   The list of selected langauges.
 * @param $module
 *   The selected module option.
 * @return array
 *   The $form array.
 */
function mail_edit_list_filter($languages_selected, $module, $language_list) {
  if (count($language_list) > 1) {
    $form['languages_selected'] = array(
      '#type'          => 'select',
      '#title'         => t('Languages'),
      '#description'   => t('Select the languages to display.'),
      '#options'       => $language_list,
      '#default_value' => array_keys($languages_selected),
      '#multiple'      => TRUE,
    );
  }

  $module_list = _mail_edit_module_list(TRUE);
  $options['-enabled'] = t('- Display enabled modules only -');
  $options['-all'] = t('- Display all modules -');
  $query = db_select('mail_edit_registry', 'mer')
    ->fields('mer', array('module'))
    ->distinct()
    ->orderBy('mer.module');
  foreach ($query->execute()->fetchCol() as $mod) {
    $options[$mod] = isset($module_list[$mod]) ? $module_list[$mod] : check_plain($mod);
  }

  $form['module'] = array(
    '#type'          => 'select',
    '#title'         => t('Module'),
    '#description'   => t('Select a module.'),
    '#options'       => $options,
    '#default_value' => $module,
  );

  $form['submit'] = array(
    '#type'  => 'item',
    '#value' => '',
  );
  $form['submit']['filter'] = array(
    '#type'  => 'submit',
    '#value' => t('Filter'),
  );
  $form['language_list'] = array(
    '#type'  => 'hidden',
    '#value' => serialize($language_list),
  );

  $form['#theme'] = 'mail_edit_list_filter';
  return $form;
}

/**
 * Create $form array with filtered list.
 *
 * @param $languages
 *   The languages to include.
 * @param $module
 *   The module option to restrict the list with.
 * @return array
 *   The $form array.
 */
function mail_edit_list_filtered_form($languages_selected, $module, $display_language_names) {
  _mail_edit_key_registry_rebuild();

  $header = _mail_edit_list_get_header($languages_selected, $display_language_names);
  $query = db_select('mail_edit_registry', 'mer')->extend('PagerDefault')->limit(50)
    ->fields('mer', array('id', 'module', 'mailkey', 'description'))
    ->orderBy('mer.module')
    ->orderBy('mer.mailkey');

  switch ($module) {
    case '-all':
      break;

    case '-enabled':
      $enabled_modules = array_keys(_mail_edit_module_list());
      $query->condition('mer.module', $enabled_modules, 'IN');
      break;

    default:
      $query->condition('mer.module', $module);
      break;
  }

  $form['templates']['#tree'] = TRUE;
  $form['#cache'] = TRUE;

  $form['header'] = array(
    '#type'  => 'value',
    '#value' => $header,
  );

  $form['templates'] = array();
  foreach ($query->execute() as $template) {
    // Add all translated languages to template.
    $query2 = db_select('mail_edit', 'me')
      ->fields('me', array('language'))
      ->condition('me.id', $template->id);
    foreach ($query2->execute() as $template2) {
      $template->languages[$template2->language] = TRUE;
    }
    $form['templates'][$template->id] = _mail_edit_list_row($template, $languages_selected, $display_language_names);
  }

  $form['languages'] = array(
    '#type'  => 'value',
    '#value' => $languages_selected,
  );

  $form['#theme'] = 'mail_edit_table';
  $form['pager'] = array('#markup' => theme('pager'));
  return $form;
}

/**
 * Build the header for the mail edit table.
 *
 * @param array $languages_selected
 *   The languages to include.
 * @param bool $display_language_names
 *   Whether to display the name of the langauge.
 * @return array
 */
function _mail_edit_list_get_header($languages_selected, $display_language_names) {
  $header = array(
    array('data' => t('Module')),
    array('data' => t('Mailkey')),
  );

  foreach ($languages_selected as $lang_name) {
    $header[] = array('data' => ($display_language_names ? $lang_name : t('Operations')));
  }
  return $header;
}

/**
 * Build one row of the mail edit table.
 *
 * @param $template
 *   The mail template for this row.
 * @param $languages
 *   The languages to include.
 * @return array
 */
function _mail_edit_list_row($template, $languages, $display_language_names) {
  $module_list = _mail_edit_module_list(TRUE);
  $form['module'] = array(
    '#markup' => isset($module_list[$template->module]) ? $module_list[$template->module] : check_plain($template->module),
  );

  $form['id'] = array(
    '#markup' => check_plain($template->id),
  );
  $form['mailkey'] = array(
    '#markup' => check_plain($template->mailkey),
  );
  $form['description'] = array(
    '#markup' => filter_xss_admin($template->description),
  );
  $form['operations'] = array(
    '#markup' => '',
  );
  foreach ($languages as $lang_code => $lang_name) {
    $variable = array('@language' => $lang_code);
    if ($display_language_names) {
      $link_text = t(isset($template->languages[$lang_code]) ? 'Edit @language' : 'Add @language', $variable);
    }
    else {
      $link_text = t(isset($template->languages[$lang_code]) ? 'Edit' : 'Add');
    }
    $form[$lang_code] = array(
      '#markup' => l($link_text, 'admin/config/system/mail-edit/' . $template->id . '/' . $lang_code),
    );
  }
  return $form;
}

/**
 * The submit handler for mail_edit_list_form().
 *
 * @param $form
 * @param $form_state
 */
function mail_edit_list_form_submit($form, &$form_state) {
  $language_list = unserialize($form_state['values']['language_list']);
  $_SESSION['mail_edit']['module'] = $form_state['values']['module'];
  if (isset($form_state['values']['languages_selected'])) {
    $_SESSION['mail_edit']['languages_selected'] = array_intersect_key($language_list, $form_state['values']['languages_selected']);
  }
}

/**
 * The theme function for the basic filter form.
 *
 * @param $variables
 * @return string
 *
 * @ingroup themeable
 */
function theme_mail_edit_list_filter($variables) {
  $form = $variables['form'];
  $output = '<div id="mail-edit-filter">';
  foreach (element_children($form) as $key) {
    $attributes = drupal_attributes(array(
      'id'    => 'mail-edit-' . str_replace('_', '-', $key) . '-filter',
      'class' => 'mail-edit-filter',
    ));
    $output .= "<div $attributes>";
    $output .= drupal_render($form[$key]);
    $output .= '</div>';
  }
  $output .= '</div>';

  return $output;
}

/**
 * The theme function for the mail edit table.
 *
 * @param $variables
 * @return string
 *
 * @ingroup themeable
 */
function theme_mail_edit_table($variables) {
  $form = $variables['form'];
  $header = $form['header']['#value'];
  $languages = $form['languages']['#value'];

  foreach (element_children($form['templates']) as $key) {
    // Build the table row.
    $row = array();
    $row['data'][] = array('data' => drupal_render($form['templates'][$key]['module']));
    $row['data'][] = array('data' => drupal_render($form['templates'][$key]['mailkey']) . '<div class="description">' . drupal_render($form['templates'][$key]['description']) . '</div>');
    foreach ($languages as $lang_code => $lang_name) {
      $row['data'][] = array('data' => drupal_render($form['templates'][$key][$lang_code]), 'class' => 'mail-edit-table-' . $lang_code);
    };
    $row['data'][] = array('data' => drupal_render($form['templates'][$key]['operations']));
    $rows[] = $row;
  }

  if (empty($rows)) {
    $rows[] = array(array(
      'data'    => t('No templates available for override.'),
      'colspan' => count($header),
    ));
  }
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'mail-edit-table')));
  if ($form['pager']) {
    $output .= drupal_render($form['pager']);
  }
  return $output;
}

/**
 * Builds the list of modules that expose mailkeys.
 *
 * @param bool $all
 *   Return all modules if TRUE, or only enabled ones otherwise.
 * @return array
 */
function _mail_edit_module_list($all = FALSE) {
  static $modules;

  if (isset($modules[$all])) {
    return $modules[$all];
  }

  $modules[$all] = array();
  $query = db_select('system', 's')
    ->fields('s', array('name', 'info'))
    ->condition('s.type', 'module')
    ->orderBy('s.name');
  if (!$all) {
    $query->condition('s.status', 1);
  }
  foreach ($query->execute() as $module) {
    $info = unserialize($module->info);
    $modules[$all][$module->name] = $info['name'];
  }
  return $modules[$all];
}

function mail_edit_trans($form, $form_status, $id, $lang) {
  _mail_edit_include();
  $query = db_select('mail_edit_registry', 'mer')
    ->fields('mer')
    ->condition('mer.id', $id);
  $mailkey_obj = $query->execute()->fetchObject();
  $mailkey = $mailkey_obj->mailkey;

  $query = db_select('mail_edit', 'me')
    ->fields('me')
    ->condition('me.id', $id)
    ->condition('me.language', $lang);
  $template = $query->execute()->fetch();
  // If template is set, then we need to update, if its not set then we need to insert.
  $update = (bool) $template;
  $language = new stdClass();
  $language->language = $lang;
  $mail_data = module_invoke($mailkey_obj->module, 'mail_edit_text', $mailkey_obj->mailkey, $language);
  if (!isset($template->subject)) {
    $template->subject = $mail_data['subject'];
  }
  if (!isset($template->body)) {
    $template->body = $mail_data['body'];
  }

  $language_list = mail_edit_language_list();
  $variables = array('%mailkey' => $mailkey, '%language' => $language_list[$lang]);
  if (count($language_list) > 1) {
    $message = t(($update
      ? 'Update %mailkey template for %language'
      : 'Create new %mailkey template for %language'), $variables);
  }
  else {
    $message = t(($update
      ? 'Update %mailkey template'
      : 'Create new %mailkey template'), $variables);
  }
  drupal_set_title($message, PASS_THROUGH);

  $form = array();
  $form['update'] = array(
    '#type'  => 'hidden',
    '#value' => $update,
  );
  $form['language'] = array(
    '#type'  => 'hidden',
    '#value' => $lang,
  );
  $form['id'] = array(
    '#type'  => 'hidden',
    '#value' => $id,
  );
  $form['description'] = array(
    '#title'         => t('Description'),
    '#type'          => 'textfield',
    '#default_value' => isset($template->description) ? $template->description : (isset($mailkey_obj->description) ? filter_xss($mailkey_obj->description, array()) : ''),
  );
  $form['mail'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Template'),
    '#collapsible'   => FALSE,
  );
  $form['mail']['subject'] = array(
    '#title'         => t('Subject'),
    '#type'          => 'textfield',
    '#default_value' => isset($template->subject) ? $template->subject : '',
  );
  $form['mail']['body'] = array(
    '#title'         => t('Body'),
    '#type'          => 'textarea',
    '#default_value' => isset($template->body) ? $template->body : '',
    '#rows' => 15,
  );

  if (module_exists('token')) {
    $token_types = module_invoke($mailkey_obj->module, 'mail_edit_token_types', $mailkey_obj->id);
    $form['mail']['token_tree'] = array(
      '#theme' => 'token_tree',
      '#token_types' => (empty($token_types) ? array() : $token_types),
      '#show_restricted' => ($mailkey_obj->module == 'user'),
    );
    drupal_add_css(drupal_get_path('module', 'mail_edit') . '/mail_edit.admin.css');
  }
  else {
    $form['mail']['token_tree'] = array(
      '#markup' => '<p>' . t('Enable the <a href="@drupal-token">Token module</a> to view the available token browser.', array('@drupal-token' => 'http://drupal.org/project/token')) . '</p>',
    );
  }

  $form['op'] = array(
    '#type'   => 'submit',
    '#value'  => $update ? t('Update template') : t('Save new template'),
    '#submit' => array('mail_edit_trans_save'),
  );
  if ($update) {
    $form['delete'] = array(
      '#type'   => 'submit',
      '#value'  => t('Remove @language translation', array('@language' => $language_list[$lang])),
      '#submit' => array('mail_edit_trans_delete'),
    );
  }
  return $form;
}

function mail_edit_trans_delete($form, &$form_state) {
  $success = db_delete('mail_edit')
    ->condition('id', $form_state['values']['id'])
    ->condition('language', $form_state['values']['language'])
    ->execute();
  $language_list = mail_edit_language_list();
  $variables = array(
    '%language' => $language_list[$form_state['values']['language']],
    '%id' => $form_state['values']['id'],
  );
  drupal_set_message(t($success
    ? '%language translation of %id has been removed.'
    : 'Failed to remove %language translation of %id.', $variables));
  $form_state['redirect'] = 'admin/config/system/mail-edit';
}

function mail_edit_trans_save($form, &$form_state) {
  $values = $form_state['values'];
  $update = $values['update'];
  if ($update) {
    $success = db_update('mail_edit')
      ->fields(array(
        'description' => $values['description'],
        'subject' => $values['subject'],
        'body' => $values['body'],
      ))
      ->condition('id', $values['id'])
      ->condition('language', $values['language'])
      ->execute();
  }
  else {
    $success = TRUE;
    db_insert('mail_edit')
      ->fields(array(
        'id' => $values['id'],
        'language' => $values['language'],
        'description' => $values['description'],
        'subject' => $values['subject'],
        'body' => $values['body'],
      ))
      ->execute();
  }
  $language_list = mail_edit_language_list();
  $variables = array('%id' => $values['id'], '%language' => $language_list[$values['language']]);
  if ($success) {
    drupal_set_message(t('Saved the %id template for the %language translation.', $variables));
  }
  $form_state['redirect'] = 'admin/config/system/mail-edit';
}

/**
 * Builds the registry of exposed mailkeys, keeps track of which module
 * exposed them.
 *
 * @return unknown
 */
function _mail_edit_key_registry_rebuild() {
  // Fetch exposed mailkeys.
  $hook = 'mailkeys';
  _mail_edit_include();
  module_implements($hook, FALSE, TRUE);
  $mailkeys = array();
  foreach (module_implements($hook) as $module) {
    $function = $module . '_' . $hook;
    $result = $function();
    if (isset($result)) {
      $mailkeys[$module] = $result;
    }
  }
  // Find out if we already have record of the exposed keys.
  $query = db_select('mail_edit_registry', 'mer')
    ->fields('mer');
  foreach ($query->execute() as $row) {
    if (isset($mailkeys[$row->module][$row->mailkey])) {
      unset($mailkeys[$row->module][$row->mailkey]);
    }
  }
  // Insert any new mailkeys into our registry table.
  foreach ($mailkeys as $module => $keys) {
    foreach ($keys as $key => $description) {
      db_insert('mail_edit_registry')
        ->fields(array(
          'id' => $module . '_' . $key,
          'module' => $module,
          'mailkey' => $key,
          'description' => $description,
        ))
        ->execute();
    }
  }
}

/**
 * Return an array of all the languages in the system.
 *
 * @return array
 */
function mail_edit_language_list() {
  if (module_exists('locale')) {
    $language_list = locale_language_list('name', TRUE);
  }
  else {
    foreach (array_keys(language_list()) as $key) {
      $language_list[$key] = $key;
    }
  }
  return $language_list;
}

